local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local Assert = BaikCore.Assert
local Log = BaikCore.Log

-- Add Module
local ChatCommand = BaikCore:NewModule("ChatCommand", "AceConsole-3.0")
BaikCore.ChatCommand = ChatCommand

-- Constants
local COMMAND = "baik"

-- Private Variables
local commands = {}
local helps = {}

-- Private Methods
local function PrintHelp()
    print("Baik Addon Chat Commands")
    print("  /baik <command>")
    for command, _ in pairs(commands) do
        print(string.format("    %s : %s", command, helps[command]))
    end
end

-- Public API
function ChatCommand:Register(command, func, help)
    Assert:String(command)
    Assert:Function(func)
    Assert:String(help)
    Assert:Null(commands[command], "Command already registered")

    commands[command] = func
    helps[command] = help
end

function ChatCommand:Deregister(command)
    Assert:String(command)

    if not commands[command] then
        Log:w("%s is not a registered command", command)
    else
        table.remove(commands, command)
    end

    if not helps[command] then
        Log:w("%s is not a registered command", command)
    else
        table.remove(helps, command)
    end
end

function ChatCommand:OnCommand(args)
    if not args or not args:match("%S") then
        PrintHelp()
        return
    end
    Assert:String(args)
    if commands[args] then
        commands[args]()
    end
end

-- Ace3 Callbacks
function ChatCommand:OnInitialize()
end

function ChatCommand:OnEnable()
    self:RegisterChatCommand(COMMAND, "OnCommand")
    self:Register("help", PrintHelp, "Display help options")
end

function ChatCommand:OnDisable()
    self:UnregisterChatCommand(COMMAND)
end
