local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local Assert = BaikCore.Assert

-- Add Module to BaikCore
local Database = BaikCore:NewModule("Database")
BaikCore.Database = Database

-- Constants
local DB_NAME = "BaikCoreDB"

-- Private Variable
local defaults = {
    profile = {
    }
}

-- Public API
function Database:Get(name, default)
    Assert:String(name)
    local value = self.db.profile[name]

    return value == nil and default or value
end

function Database:Raw()
    return self.db
end

function Database:Set(name, value)
    Assert:String(name)

    self.db.profile[name] = value
end

function Database:InitIfEmpty(name, value)
    Assert:String(name)

    -- Adds value to defaults
    defaults.profile[name] = value

    -- Loads default if empty
    if self.db.profile[name] == nil then
        self.db.profile[name] = value
    end

    return self.db.profile[name]
end

function Database:RefreshConfig()
    BaikCore.Reload:Enable()
end

-- Ace3 Callbacks
function Database:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New(DB_NAME)
end

function Database:OnEnable()
    self.db:RegisterDefaults(defaults)
    self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
    self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
    self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")
end

function Database:OnDisable()
end
