local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local Assert = BaikCore.Assert
local Database = BaikCore.Database
local Debug = BaikCore.Debug
local Log = BaikCore.Log
local OrderNumber = BaikCore.OrderNumber

-- Add Module to BaikCore
local Option = BaikCore:NewModule("Option")
BaikCore.Option = Option

local orderNumber = OrderNumber:Create(1)

-- Constants
local options = {
    name = "Baik",
    handler = BaikCore,
    type = "group",
    args = {
        general = {
            name = "General",
            type = "group",
            order = 0,
            childGroups = "tab",
            args = {
            }
        },
    }
}

Option.NAME = "general"

-- Public API
function Option:AddCategory(category, opts)
    Assert:String(category)
    if not options.args[category] then
        options.args[category] = opts
    end
end

function Option:AddOption(category, name, opt)
    Assert:String(category)
    Assert:String(name)
    Assert:NotNull(options.args[category])
    Assert:NotNull(options.args[category].args)

    options.args[category].args[name] = opt
end

function Option:SetHidden(value, category, name)
    Assert:Boolean(value)
    Assert:String(category)
    local opt = options.args[category]
    Assert:NotNull(opt)
    if name then
        Assert:String(name)
        Assert:NotNull(opt.args)
        Assert:NotNull(opt.args[name])
        opt = opt.args[name]
    end
    opt.hidden = value
end

function Option:GeneralOrderNumber()
    return orderNumber:GetNext()
end

-- Ace3 Callbacks
function Option:OnInitialize()
    options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(Database:Raw())
    options.args.profiles.order = -1
end

function Option:OnEnable()
    LibStub("AceConfig-3.0"):RegisterOptionsTable(BaikCore.NAME, options)
    LibStub("AceConfigDialog-3.0"):AddToBlizOptions(BaikCore.NAME, BaikCore.NAME)
end

function Option:OnDisable()
end
