local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local Assert = BaikCore.Assert
local Database = BaikCore.Database
local Log = BaikCore.Log
local Option = BaikCore.Option

-- Add Module
local Zoom = BaikCore:NewModule("Zoom")
BaikCore.Zoom = Zoom

-- Constants
local ZOOM_COORD = {0.1, 0.9, 0.1, 0.9}
local NORM_COORD = {0, 1, 0, 1}

local OPTION_NAME = "zoom_style"

local SUFFIX_DB_ZOOM = "%s_zoom"

local PREFIX_DB = "global"
local GLOBAL_ZOOM_DB = string.format(SUFFIX_DB_ZOOM, PREFIX_DB)

Zoom.DEFAULT = false

-- Private Variables
local zoomFunc = {}

-- Private Methods
local function GetDbZoom(dbPrefix)
    return string.format(SUFFIX_DB_ZOOM, dbPrefix)
end

local function GetZoomed(dbPrefix)
    local globalZoom = Database:Get(GLOBAL_ZOOM_DB)
    local dbZoom = GetDbZoom(dbPrefix)

    return globalZoom or Database:Get(dbZoom)
end

local function GetCoords(dbPrefix)
    local zoomed = GetZoomed(dbPrefix)

    return zoomed and ZOOM_COORD or NORM_COORD
end

local function ZoomButton(btn, coords)
    btn.icon:SetTexCoord(unpack(coords))
    btn:GetCheckedTexture():SetTexCoord(unpack(coords))
end

local function ZoomTexture(texture, coords)
    texture:SetTexCoord(unpack(coords))
end

local function Setup()
    for _, func in pairs(zoomFunc) do
        func()
    end
end

local function GetOpts()
    return {
        name = "Icon",
        type = "group",
        args = {
            zoom = {
                name = "Zoom",
                type = "toggle",
                get = function(...)
                    return Database:Get(GLOBAL_ZOOM_DB)
                end,
                set = function(self, value, ...)
                    Database:Set(GLOBAL_ZOOM_DB, value)
                    Setup()
                end,
            }
        }
    }
end

-- Zoom API
function Zoom:SetZoomButton(dbPrefix, btn)
    Assert:Table(btn)
    Assert:String(dbPrefix)

    local coords = GetCoords(dbPrefix)
    ZoomButton(btn, coords)
end

function Zoom:SetZoomButtons(dbPrefix, btnTable)
    Assert:Table(btnTable)
    Assert:String(dbPrefix)

    local coords = GetCoords(dbPrefix)
    for _, btn in pairs(btnTable) do
        ZoomButton(btn, coords)
    end
end

function Zoom:SetZoomTexture(dbPrefix, tex)
    Assert:Table(tex)
    Assert:String(dbPrefix)

    local coords = GetCoords(dbPrefix)
    ZoomTexture(tex, coords)
end

function Zoom:SetZoomTextures(dbPrefix, texTable)
    Assert:Table(texTable)
    Assert:String(dbPrefix)

    local coords = GetCoords(dbPrefix)
    for _, tex in pairs(texTable) do
        ZoomTexture(tex, coords)
    end
end

function Zoom:InitDb(dbPrefix)
    Assert:String(dbPrefix)

    Database:InitIfEmpty(GetDbZoom(dbPrefix), self.DEFAULT)
end

function Zoom:GetOpts(dbPrefix, SetZoomFunc)
    Assert:String(dbPrefix)
    Assert:Function(SetZoomFunc)

    zoomFunc[dbPrefix] = SetZoomFunc
    local dbZoom = GetDbZoom(dbPrefix)
    return {
        name = "Zoom",
        type = "toggle",
        disabled = function(...)
            return Database:Get(GLOBAL_ZOOM_DB)
        end,
        get = function(...)
            return Database:Get(dbZoom)
        end,
        set = function(self, value, ...)
            Database:Set(dbZoom, value)
            SetZoomFunc(value)
        end,
    }
end

-- Ace3 Callbacks
function Zoom:OnInitialize()
    Log:v("Zoom_OnInitialize")
    Database:InitIfEmpty(GLOBAL_ZOOM_DB, self.DEFAULT)
    Option:AddOption(Option.NAME, OPTION_NAME, GetOpts())
end

function Zoom:OnEnable()
    Log:v("Zoom_OnEnable")
    Setup()
end

function Zoom:OnDisable()
    Log:v("Zoom_OnDisable")
end
